#!/usr/bin/python
# Copyright (c) 2004-2016 Bright Computing Holding BV. All Rights Reserved.
#
# This software is the confidential and proprietary information of
# Bright Computing Holding BV ("Confidential Information").  You shall not
# disclose such Confidential Information and shall use it only in
# accordance with the terms of the license agreement you entered into
# with Bright Computing Holding BV or its subsidiaries.

from __future__ import print_function

import os


class LinuxDistro(object):

    def __init__(self, path=''):
        self.path = path
        self.basedist = None
        self.dist = None
        self.distu = None
        self.version = None
        self.fulldist = None
        self.fulldistu = None
        self.update = None
        self.isrhel5 = False
        self.isrhel6 = False
        self.isrhel7 = False
        self.issles11 = False
        self.issles12 = False
        self.issles15 = False
        self.isubuntu = False
        self.distunsupported = None
        self.getLinuxDistro()

    def getLinuxDistro(self):
        if os.access('%s/etc/SuSE-release' % self.path, os.R_OK):
            self.basedist = 'suse'
            with open('%s/etc/SuSE-release' % self.path) as f:
                for line in f:
                    line = line.replace('\n', '')
                    fields = line.split(' = ')
                    # test for sles
                    if line.startswith('SUSE Linux Enterprise Server'):
                        self.dist = 'sles'
                    if len(fields) >= 2:
                        if fields[0] == 'VERSION':
                            self.version = fields[1].strip()
                            if self.version == '11':
                                self.issles11 = True
                            elif self.version == '12':
                                self.issles12 = True
                        elif fields[0] == 'PATCHLEVEL':
                            self.update = fields[1].strip()
        elif os.access('%s/etc/oracle-release' % self.path, os.R_OK):
            self.basedist = 'redhat'
            with open('%s/etc/oracle-release' % self.path) as f:
                line = f.readline()
            line = line.replace('\n', '')
            # test for oel
            if line.startswith('Oracle Linux'):
                self.dist = 'oel'
                self.redhatVersion(line)
            else:
                try:
                    ufile = open('%s/etc/oracle-release' % self.path)
                    self.distunsupported = ufile.read().strip('\n')
                    ufile.close()
                except Exception as err:  # noqa: F841
                    pass
                self.dist = 'unsupported'
                self.basedist = line
                self.redhatVersion(line)
        elif os.access('%s/etc/redhat-release' % self.path, os.R_OK):
            with open('%s/etc/redhat-release' % self.path) as f:
                line = f.readline()
            line = line.replace('\n', '')
            # test for rhel
            if line.startswith('Red Hat Enterprise Linux'):
                self.dist = 'rhel'
                self.basedist = 'redhat'
            # test for sl
            elif line.startswith('Scientific Linux'):
                self.dist = 'sl'
                self.basedist = 'redhat'
            # test for centos
            elif line.startswith('CentOS'):
                self.dist = 'centos'
                self.basedist = 'redhat'
            # test for rocky
            elif line.startswith('Rocky'):
                self.dist = 'rocky'
                self.basedist = 'redhat'
            else:
                try:
                    with open('%s/etc/redhat-release' % self.path) as ufile:
                        self.distunsupported = ufile.read().strip('\n')
                except Exception as err:  # noqa: F841
                    pass
                self.dist = 'unsupported'
                self.basedist = line
            self.redhatVersion(line)
        elif os.access('%s/etc/os-release' % self.path, os.R_OK):
            with open('%s/etc/os-release' % self.path) as f:
                lines = f.readlines()
            for ln in lines:
                ln.strip('\n')
                if 'ID=ubuntu' in ln:
                    self.isubuntu = True
                    self.dist = 'ubuntu'
                    self.basedist = 'ubuntu'
                elif 'ID="sles"' in ln:
                    self.basedist = 'suse'
                    self.dist = 'sles'
            for ln in lines:
                ln.strip('\n')
                if 'VERSION_ID' in ln:
                    if 'sles' in self.dist:
                        self.version = (ln.split('=')[1]).strip().replace('"', '').split('.')[0]
                        self.update = (ln.split('=')[1]).strip().replace('"', '').split('.')[1]
                    else:
                        self.version = (ln.split('=')[1]).strip().replace('"', '').replace('.', '')

        if self.dist is not None:
            self.distu = self.dist.upper()
            if self.basedist == 'suse':
                if self.update is not None:
                    if self.update > 0:
                        self.fulldistu = self.distu + str(self.version) + 'sp' + str(self.update)
                        self.fulldist = self.dist + str(self.version) + 'sp' + str(self.update)
                else:
                    self.fulldistu = self.distu + str(self.version)
                    self.fulldist = self.dist + str(self.version)
                if self.version == '15':
                    self.issles15 = True
            else:
                self.fulldistu = self.distu + str(self.version)
                self.fulldist = self.dist + str(self.version)

    def redhatVersion(self, string):
        fields = string.split()
        if 'release' in fields:
            index = fields.index('release')
            if len(fields) > index + 1:
                version = fields[index + 1].strip()
        if 'Update' in fields:
            index = fields.index('Update')
            if len(fields) > index + 1:
                self.update = fields[index + 1].replace(')', '').strip()
        if '.' in version:
            fields = version.split('.')
            self.version = fields[0].strip()
            self.update = fields[1].strip()
        else:
            self.version = version
        if self.version == '5':
            self.isrhel5 = True
        elif self.version == '6':
            self.isrhel6 = True
        elif self.version == '7':
            self.isrhel7 = True
        elif self.version == '8':
            self.isrhel8 = True

    def isValid(self):
        return self.basedist in ['redhat', 'suse', 'ubuntu']

    @property
    def validbasedist(self):
        return self.isValid()

    def show(self):
        print('base distribution:          %s' % self.basedist)
        print('distribution:               %s' % self.dist)
        print('distribution (upper):       %s' % self.distu)
        print('version:                    %s' % self.version)
        print('update:                     %s' % self.update)
        print('full dist,version,update :  %s' % self.fulldist)
        print('full (upper):               %s' % self.fulldistu)
        print('valid base distribution:    %s' % self.validbasedist)
        print('is rhel5 base distribution: %s' % self.isrhel5)
        print('is rhel6 base distribution: %s' % self.isrhel6)
        print('is rhel7 base distribution: %s' % self.isrhel7)
        print('is rhel8 base distribution: %s' % self.isrhel8)
        print('is sles11 base distribution:%s' % self.issles11)
        print('is sles12 base distribution:%s' % self.issles12)
        print('is sles15 base distribution:%s' % self.issles15)
        print('is ubuntu base distribution:%s' % self.isubuntu)

    def showline(self, separator=' '):
        print(separator.join([self.basedist, self.dist, self.version, self.update or '']))

    def __str__(self):
        return ' '.join([self.basedist, self.dist, self.version, self.update or ''])


linuxDistro = LinuxDistro  # (med) For compatibility.
